/*
 *  radutmped.c - main file for radutmped
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *   Copyright (C) 2005, 2006, 2007 Predrag Malicevic <pmalic(at)develabs[dot]com>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <libgen.h>
#include "radutmped.h"

void usage(char *progname)
{
	fprintf(stderr, "INFO: radutmp editor v%s - Copyright (C) 2005, 2006, 2007 Predrag Malicevic et al\n\nUSAGE: %s <radutmpfile> L<login>|P<nas_port>|S<session_id>\n", RADUTMPED_VERSION, progname);
	exit(EXIT_FAILURE);
}

int main(int argc, char **argv)
{
	int fd, size;
	struct radutmp radutmp;
	unsigned int search_mode, entry_found, nas_port;
	char *file_name, *search_term, *login, *session_id, *search_mode_desc, *file_name_copy;
  char session_id_copy[SESSION_ID_LEN + 1];

	search_mode = SEARCH_UNKNOWN;
	entry_found = nas_port = 0;
	file_name = search_term = login = session_id = search_mode_desc = NULL;

	if(argc != 3)
	   usage(argv[0]);

	file_name = argv[1];
	file_name_copy = strdup(file_name);
	if(strncmp(basename(file_name_copy), RADUTMP_FILENAME, strlen(RADUTMP_FILENAME)))
	{
		fprintf(stderr, "\nERROR: filename must begin with '%s'.\n\n", RADUTMP_FILENAME);
		usage(argv[0]);
	}
	free(file_name_copy);

	search_term = argv[2];
	if(isupper(search_term[0]))
	{
		switch(search_term[0])
		{
			case 'L':
				search_mode = SEARCH_LOGIN;
				search_mode_desc = "LOGIN";
				login = search_term + 1;
				break;
			case 'P':
				search_mode = SEARCH_NAS_PORT;
				search_mode_desc = "NAS-PORT";
				nas_port = strtol(search_term + 1, (char **) NULL, 10);
				break;
			case 'S':
				search_mode = SEARCH_SESSION_ID;
				search_mode_desc = "SESSION-ID";
				session_id = search_term + 1;
				break;
			default:
				fprintf(stderr, "\nERROR: unknown search mode.\n\n");
				usage(argv[0]);
				break;
		}
	}
	else
	{
		fprintf(stderr, "\nERROR: unknown search mode.\n\n");
		usage(argv[0]);
	}

	fd = open(file_name, O_RDWR);
	if(fd >= 0)
	{
		size = read(fd, &radutmp, sizeof(struct radutmp));
		while(size == sizeof(struct radutmp))
		{
			switch(search_mode)
			{
				case SEARCH_LOGIN:
					entry_found = (!strncmp(radutmp.login, login, LOGIN_LEN));
					break;
				case SEARCH_NAS_PORT:
					if(nas_port > 0)
						entry_found = (radutmp.nas_port == nas_port);
					else
						entry_found = 0;
					break;
				case SEARCH_SESSION_ID:
					entry_found = (!strncmp(radutmp.session_id, session_id, SESSION_ID_LEN));
					break;
				default:
					entry_found = 0;
					break;
			}

			if(entry_found)
			{
				strncpy(session_id_copy, radutmp.session_id, SESSION_ID_LEN);
				session_id_copy[SESSION_ID_LEN] = '\0';

				fprintf(stdout, ">> REMOVING ENTRY FOR %s '%s' [L%s, P%u, S%s]\n", search_mode_desc, search_term + 1, radutmp.login, radutmp.nas_port, session_id_copy);
				lseek(fd, - sizeof(struct radutmp), L_INCR);
				bzero(&radutmp, sizeof(struct radutmp));
				write(fd, &radutmp, sizeof(struct radutmp));
			}
			size = read(fd, &radutmp, sizeof(struct radutmp));
		}

		close(fd);
	}
	else
	{
		fprintf(stderr, "\nERROR: could not open radutmp file '%s'.\n", file_name);
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}
